#
#   YoctOS: LPC2148 Dev Board Operating System
#        
#   (C) Jason Hunt
#   nulluser@gmail.com
#        
#   File: makefile.m

NAME = YOCTOS
TARGET = image

CC = arm-elf-gcc
LD = arm-elf-ld -v
AR = arm-elf-ar
AS = arm-elf-as
CP = arm-elf-objcopy
OD = arm-elf-objdump

# RAM_IMAGE for testing
# ROM_IMAGE for release
# Vectors will get mapped differently 
IMAGE_TYPE=RAM_IMAGE


ifeq ($(IMAGE_TYPE),RAM_IMAGE)
    LDTEMPLATE = ram_link.ld
endif

ifeq ($(IMAGE_TYPE),ROM_IMAGE)
    LDTEMPLATE = rom_link.ld
endif


# Location of system libaraies
LIBS1 = c:/winarm/arm-elf/lib/
LIBS2 = c:/winarm/lib/gcc/arm-elf/4.1.1/
LIBS = -L${LIBS1} -lm -lc -lg  -L${LIBS2} -lgcc


OPTIMIZE = -Os


# Setup C Flags

CFLAGS =   $(THUMB) -I./include -c -fno-common $(OPTIMIZE) -Wall 
CFLAGS += -Wa,-adhlns=$(subst $(suffix $<),.lst,$<) 
CFLAGS += -D$(IMAGE_TYPE) -std=gnu99

# Setup A Flags

AFLAGS = -D$(IMAGE_TYPE)


# Setup linker flags

LDFLAGS  = -Map $(TARGET).map -Tlinker/$(LDTEMPLATE) --cref

CPFLAGS = -O ihex
ODFLAGS	= -x --syms

THUMB = 
#-mthumb -mthumb-interwork 

# Setup the programmer
#LPC21ISP = lpc21isp
LPC21ISP = lpc21isp_beta
LPC21ISP_PORT = com1
LPC21ISP_BAUD = 19200
LPC21ISP_XTAL = 12000
LPC21ISP_FLASHFILE = $(TARGET).hex
# verbose output:
# LPC21ISP_DEBUG = -debug
# bootloader mode
LPC21ISP_CONTROL = -control -term


# Make Rules

all: Core

clean:
	-rm $(TARGET).bin $(TARGET).hex $(TARGET).map *.lst *.o kernel/*.lst drivers/*.lst


Core: image.bin
	@ echo "Programming"
	$(CP) $(CPFLAGS) $(TARGET).bin $(TARGET).hex
	$(LPC21ISP) $(LPC21ISP_CONTROL) $(LPC21ISP_DEBUG) $(TARGET).hex $(LPC21ISP_PORT) $(LPC21ISP_BAUD) $(LPC21ISP_XTAL)

image.bin: boot.o main.o video.o serial.o init.o font.o utility.o interrupt.o timer.o printk.o task.o system.o memory.o signal.o user.o
	@ echo "Linking"
	$(LD) $(LDFLAGS) -o $(TARGET).bin *.o $(LIBS) 

boot.o: boot/boot.S
	@ echo "Assembling boot.S"
	$(CC) -c $(AFLAGS) -o crt0.o boot/boot.S > boot.lst

system.o: kernel/system.S include/kernel/system.h
	@ echo "Assembling system"
	$(CC) -c $(AFLAGS) -o system.o kernel/system.S > system.lst

main.o: kernel/main.c
	@ echo "Compiling main"
	$(CC) $(THUMB) $(CFLAGS) kernel/main.c
	
init.o: kernel/init.c
	@ echo "Compiling init"
	$(CC) $(CFLAGS) kernel/init.c

utility.o: kernel/utility.c
	@ echo "Compiling utility"
	$(CC) $(CFLAGS) kernel/utility.c

printk.o: kernel/printk.c
	@ echo "Compiling printk"
	$(CC) $(CFLAGS) kernel/printk.c
	
task.o: kernel/task.c
	@ echo "Compiling task"
	$(CC) $(CFLAGS) kernel/task.c

signal.o: kernel/signal.c
	@ echo "Compiling signal"
	$(CC) $(CFLAGS) kernel/signal.c

		
memory.o: kernel/memory.c
	@ echo "Compiling memory"
	$(CC) $(CFLAGS) kernel/memory.c
	
	

# Drivers


font.o: drivers/font.c
	@ echo "Compiling font"
	$(CC) $(CFLAGS) drivers/font.c

video.o: drivers/video.c
	@ echo "Compiling video"
	$(CC) $(CFLAGS) drivers/video.c

serial.o: drivers/serial.c
	@ echo "Compiling serial"
	$(CC) $(CFLAGS) drivers/serial.c

interrupt.o: drivers/interrupt.c
	@ echo "Compiling interrupt"
	$(CC) $(CFLAGS) drivers/interrupt.c

timer.o: drivers/timer.c
	@ echo "Compiling timer"
	$(CC) $(CFLAGS) drivers/timer.c


# usercode

user.o: user/user.c
	@ echo "Compiling User Code"
	$(CC) $(CFLAGS) user/user.c

