/*
    YoctOS: LPC2148 Dev Board Operating System

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.        

    (C) Jason Hunt
    nulluser@gmail.ccom
     
    File: boot.S
*/
  
.section .startup                       /* Tell linker this is startup code */
.arm

    /* Operating mode bits */
    .set    MODE_FIQ,  0x11            
    .set    MODE_IRQ,  0x12           	
    .set    MODE_SVC,  0x13           	
    .set    MODE_ABT,  0x17           	
    .set    MODE_UND,  0x1B           	
    .set    MODE_SYS,  0x1F          	

    .set    IRQ_FLAG,  0x80              /* Flag for IRQ disable */
    .set    FIQ_FLAG,  0x40              /* Flag for FIQ disable */

    .set    VECT_BASE, 0x40000000       /* Vectors get mapped to here */
    .set    VECT_END,  0x40000100       /* End of correct vector area */

    .set    MEMMAP, 0xE01FC040          /* MEMMAP register */

.global _boot

_boot:
_vectors:       
    ldr     pc, map_reset
    ldr     pc, map_undef
    ldr     pc, map_swi 
    ldr     pc, map_pabt
    ldr     pc, map_dabt
    nop					
    ldr     pc, [pc, #-0x0ff0]          /* Use VIC address */
    ldr     pc, map_fiq


/* Vector map must be 64 bytes */
map_reset:  .word   reset
map_undef:  .word   fail_undef
map_swi:    .word   task_swi_isr
map_pabt:   .word   fail_pabt
map_dabt:   .word   fail_dabt
map_nop:    .word   0					
map_irq:    .word   0					
map_fiq:    .word   reset
    
reset:  


/* Copy over vectors and set memmap for ram image mode */
#if defined(RAM_IMAGE)

    /* Copy interrupt vector table to 0x4000000 */
    ldr     r1, =_vectors        /* Start of vectors */
    ldr     r2, = VECT_BASE      /* proper place */
    ldr     r3, = VECT_END       /* End of vector */

1:  cmp     r2, r3
    ldrlo   r0, [r1], #4
    strlo   r0, [r2], #4
    blo     1b

    /*  Setup memmap register */
    ldr     r0, =MEMMAP
    mov     r1, #2
    str     r1, [r0]
#endif

    /* Setup stack for each mode. Interrupts disabled in all mode  except user */

    ldr   r0, =stack_end
    msr   cpsr_c, #MODE_UND | IRQ_FLAG | FIQ_FLAG 	/* Undefined */
    mov   sp, r0
    ldr   r1, =undef_stack
    sub   r0, r0, r1

    msr   cpsr_c, #MODE_ABT | IRQ_FLAG | FIQ_FLAG 	/* Abort */
    mov   sp, r0
    ldr   r1, =abort_stack
    sub   r0, r0, r1

    msr   cpsr_c, #MODE_FIQ | IRQ_FLAG | FIQ_FLAG 	/* FIQ */
    mov   sp, r0	
    ldr   r1, =fiq_stack
   	sub   r0, r0, r1

    msr   cpsr_c, #MODE_IRQ | IRQ_FLAG | FIQ_FLAG 	/* IRQ */
    mov   sp, r0
    ldr   r1, =irq_stack
    sub   r0, r0, r1

    msr   cpsr_c, #MODE_SVC | IRQ_FLAG | FIQ_FLAG 	/* Supervisor */
    mov   sp, r0
    ldr   r1, =svc_stack
    sub   r0, r0, r1

    msr   cpsr_c, #MODE_SYS 	/* System mode, Interrupts enabled */
    mov   sp, r0


#if defined(ROM_IMAGE)
	/* Copy .data section  */
    ldr     r1, =_etext
    ldr     r2, =_data
    ldr     r3, =_edata

2:  cmp     r2, r3
    ldrlo   r0, [r1], #4
    strlo   r0, [r2], #4
    blo     2b

#endif

	/* Clear .bss section to zero  */
    mov     r0, #0
    ldr     r1, =_bss_start
    ldr     r2, =_bss_end

3:	cmp     r1, r2
    strlo   r0, [r1], #4
    blo     3b

    bl       main       /* Branch to C code */
    b       .           /* If main returns */


.end

