/*
    YoctOS: LPC2148 Dev Board Operating System

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.        

    (C) Jason Hunt
    nulluser@gmail.com
    
    File: interrupt.c
*/

#include <core.h>



/* Enable IRQ interrupts */
void interrupt_enable_irq(void)
{
    set_cpsr(get_cpsr() & ~(IRQ_MASK | FIQ_MASK));
}
/* End of interrupt_enable_irq */


/* Disable IRQ interrupts */
void interrupt_disable_irq(void)
{
    set_cpsr(get_cpsr() | IRQ_MASK | FIQ_MASK);
}
/* End of Disable IRQ interrupts */


/* Default Interrupt Handeler */
void interrupt_default_irq(void) 
{
    VICVectAddr = 0;
    panic("Interrupt default irq");
}
/* End of interrupt_default_irq */


/* Setup the VIC 
 Dependencies: None
*/
void interrupt_init(void)
{
    VICIntEnClr = 0xffffffff;
    VICVectAddr = 0;
    VICIntSelect = 0;

    dword *vect_addr = (dword *) &VICVectAddr0;
    dword *vect_cntl = (dword *) &VICVectCntl0;

    byte i = 0;

    // Clear addresses and disable all slots
    for ( i = 0; i < VIC_SIZE; i++ )
    {
    	vect_addr[i] = 0;	
    	vect_cntl[i] = 0;
    }

    VICDefVectAddr = (dword)interrupt_default_irq;   
}
/* End of interrupt_init */


/* Hook an interrupt */
byte interrupt_hook(dword interrupt_num, void *isr)
{
    enter_critical();
      
    // Disable the interrupt we are hooking
    VICIntEnClr = 1 << interrupt_num;	
    
    // Setup array of slots
    dword *vect_addr = (dword *) &VICVectAddr0;
    dword *vect_cntl = (dword *) &VICVectCntl0;

    byte i = 0;
    
    // Search for free slot
    while(vect_addr[i] != 0 && i < VIC_SIZE) i++;
    
    // No free slots    
    if (i == VIC_SIZE) 
    {
        leave_critical();
        return(FAIL);
    }

    vect_addr[i] = (dword)isr;
    vect_cntl[i] = (dword)(IRQ_SLOT_EN | interrupt_num);

    VICIntEnable = 1 << interrupt_num;
    
    leave_critical();
    return(OK);
}
/* End of interrupt_hook */











