/*
    YoctOS: LPC2148 Dev Board Operating System

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.        

    (C) Jason Hunt
    nulluser@gmail.com
  
    File: timer.c
*/

#include <core.h>
   
dword ticks = 0;        // Tick counter



/* Set the timer 
 Dependencies: None
*/
void timer_init( void )
{
    ticks = 0;

    T0MR0 = (PCLK / 1000) * timer_0_interval;
    T0MCR = 3;           //  Interrupt and Reset on MR0 

    // Connected to VIC slot
   /*( if (interrupt_hook(TIMER0_INT, (void *)task_tick_isr)) 
    {
        // error;
    }
    */

}
/* End of timer_init */

/* Enable the timer */
void timer_enable( byte t, void * isr )
{
    if ( t == 0 )
    {	
        if (interrupt_hook(TIMER0_INT, isr)) 
            panic("Unable to hook timer0 interrupt");
        
        T0TCR = 1; 
    } else
    if ( t == 1 )	
    {
        if (interrupt_hook(TIMER1_INT, isr)) 
            panic("Unable to hook timer1 interrupt");
        
        T1TCR = 1; 
    }
    else
    {
        panic("Attempting to enable unknown timer");
        // error
    }
    
}
/* End of timer_enable */

void timer_disable( byte t, void * isr )
{
    if ( t == 0 )
    {	
//        if (interrupt_hook(TIMER0_INT, isr)) 
//            panic("Unable to hook timer0 interrupt");
        
        T0TCR = 0; 
    } else
    if ( t == 1 )	
    {
//        if (interrupt_hook(TIMER1_INT, isr)) 
//            panic("Unable to hook timer1 interrupt");
        
        T1TCR = 0; 
    }
    else
    {
        panic("Attempting to disable unknown timer");
        // error
    }
    
}
/* End of timer_enable */


/* Return tick counter */
dword timer_get_ticks( void )
{
    enter_critical();    
    
    dword ret = ticks;

    leave_critical();      
        
    return(ret);
}
/* End of task_get_ticks */

