/*
    YoctOS: LPC2148 Dev Board Operating System

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.        

    (C) Jason Hunt
    nulluser@gmail.com
    
    File: video.h
*/

#ifndef VIDEO_H
#define VIDEO_H

#define	PHILLIPS
//#define	EPSON


#define LCD_CONTRAST    0x42

#define VIDEO_RED       0xF00
#define VIDEO_GREEN     0x0F0
#define VIDEO_BLUE      0x00F
#define VIDEO_WHITE     0xFFF
#define VIDEO_BLACK     0x000

// Display size
#define VIDEO_X         132
#define VIDEO_Y         132

// Font size
#define FONT_X          6
#define FONT_Y          8

// Text area size 
#define TEXT_X          21
#define TEXT_Y          16

#define TEXT_ROWS (FONT_Y * TEXT_Y)       // Scrolling window in lcd rows


void video_init(void);
void video_clear(void);

void video_putch(byte ch);

void video_set_pos(byte x, byte y);
void video_get_pos(byte *x, byte *y);

void video_filled_box(byte x, byte y, byte xs, byte ys, word color);
void video_put_pixel(byte x, byte y, word color );

void video_set_contrast(byte c);
void video_set_background(word c);
word video_get_background(void);

void video_set_foreground(word c);
word video_get_foreground(void);


// Epson commands
#ifdef EPSON
  #define DISPON    0xAF
  #define DISOFF    0xAE
  #define DISNOR    0xA6
  #define DISINV    0xA7
  #define SLPIN     0x95
  #define SLPOUT    0x94
  #define COMSCN    0xBB
  #define DISCTL    0xCA
  #define PASET     0x75
  #define CASET     0x15
  #define DATCTL    0xBC
  #define RGBSET8   0xCE
  #define RAMWR     0x5C
  #define RAMRD     0x5D
  #define PTLIN     0xA8
  #define PTLOUT    0xA9
  #define RMWIN     0xE0
  #define RMWOUT    0xEE
  #define ASCSET    0xAA
  #define SCSTART   0xAB
  #define OSCON     0xD1
  #define OSCOFF    0xD2
  #define PWRCTR    0x20
  #define VOLCTR    0x81
  #define VOLUP     0xD6
  #define VOLDOWN   0xD7
  #define TMPGRD    0x82
  #define EPCTIN    0xCD
  #define EPCOUT    0xCC
  #define EPMWR     0xFC
  #define EPMRD     0xFD
  #define EPSRRD1   0x7C
  #define EPSRRD2   0x7D
  #define NOP       0x25
#endif

// Phillips commands
#ifdef PHILLIPS
  #define NOP      0x00
  #define SWRES     0x01
  #define BSTRON    0x03
  #define SLEEPIN   0x10
  #define SLEEPOUT	0x11
  #define NORON		0x13
  #define INVOFF	0x20
  #define INVON     0x21
  #define SETCON	0x25
  #define DISPOFF   0x28
  #define DISPON    0x29
  #define CASET     0x2A
  #define PASET     0x2B
  #define RAMWR     0x2C
  #define RGBSET	0x2D
  #define SRCENB    0x33
  #define MADCTL    0x36
  #define SETEP     0x37
  #define COLMOD	0x3A
  #define DISCTR    0xB9
  #define EC		0xC0
#endif


#endif


