/*
    YoctOS: LPC2148 Dev Board Operating System

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.        

    (C) Jason Hunt
    nulluser@gmail.com
     
    File: memory.h
*/

#ifndef MEMORY_H
#define MEMORY_H

// Do not split a block if less than this will remain free afterward
#define MIN_SPLIT       128        
#define BLOCK_USED      0x80000000 // Block used flag

//#define MEMORY_DEBUG          // Define if memory testing is to be included


typedef struct block_type
{
    dword size;                 // (4 bytes) Size of the block 
    struct block_type *prev;   // (4 bytes) Pointer to previous block
    struct block_type *next;   // (4 bytes) Pointer to next block
    dword checksum;             // (4 bytes) Used to make sure the block is not corrupt
                              
} block_type;                  


void memory_init(void);
void memory_test(void);

void kfree(void * addr);
void * kmalloc(dword size);

dword memory_get_free(void);

extern dword heap_start;        // Heap region provided by linker script
extern dword heap_end;

#endif
