/*
    YoctOS: LPC2148 Dev Board Operating System

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.        

    (C) Jason Hunt
    nulluser@gmail.com
    
    File: system.h
*/

//..../..clude "core.h"

#ifndef SYSTEM_H
#define SYSTEM_H

dword get_cpsr( void );
void set_cpsr( dword v );

/* 
    Enter critical seciton 
    Disable interrupts and save old cpsr 
*/
#define enter_critical()                                                \
																		\
    dword save_cpsr = get_cpsr();                                       \
                                                                        \
    set_cpsr(save_cpsr | IRQ_MASK | FIQ_MASK);                         

/* End of enter_critical */




/* 
    Exit critical section
    Restore previous interrupt state 
*/


#define leave_critical()                                                \
                                                                        \
    set_cpsr( (get_cpsr() & ~(IRQ_MASK | FIQ_MASK)) |                   \
              (save_cpsr & (IRQ_MASK | FIQ_MASK))) ;        

/* End of leave_critical */





/*
    Save current task context into prev_task structure   
    Remeber that LR is irq or swi mode registers
    
*/

#define save_context()                                                  \
{																		\
	asm volatile                                                        \
    ("														            \
	    stmdb  sp!, {r0};												\
																		\
    	/* Get stack pointer for current task. */				        \
    	stmdb  sp,{sp}^;												\
    	sub    sp, sp, #4;											    \
    	ldmia  sp!,{r0};											    \
																		\
    	/* Save return address. */                                      \
    	stmdb  r0!, {lr};										 	    \
																		\
    	mov    lr, r0;													\
    	ldmia  sp!, {r0};											    \
																		\
    	/* Save all user mode registers. */                             \
    	stmdb  lr,{r0-lr}^;                                             \
    	sub    lr, lr, #60;										        \
																		\
    	/* Save flags  */                          						\
    	mrs    r0, spsr;												\
    	stmdb  lr!, {r0};										        \
																		\
        /* Save new task stack pointer into cur task structure */       \
    	ldr    r0, =cur_task;									        \
    	ldr    r0, [r0];										        \
    	str    lr, [r0];										        \
    ");																	\
}


/* 
    Restore context from cur_task 
*/
#define restore_context()											    \
{                                                                       \
	asm volatile                                                        \
    ("                                                                  \
        /* Get stack pointer from task structure */                     \
    	ldr    r0, =cur_task;                                           \
    	ldr    r0, [r0];                                                \
    	ldr	   lr, [r0];                                                \
																		\
        /* Restore flags */                                             \
    	ldmfd  lr!, {r0};                                           	\
    	msr    spsr, r0;                                             	\
																    	\
        /* Restore task registers */                                    \
    	ldmfd  lr, {r0-lr}^;                                        	\
                                                                        \
        /* Get return address directly from stack */                    \
    	ldr    lr, [lr, #+60];                                          \
																		\
        /* Return with flag restore */                                  \
    	subs   pc, lr, #4;                                              \
    ");																    \
}

#endif














