/*
    YoctOS: LPC2148 Dev Board Operating System

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.        

    (C) Jason Hunt
    nulluser@gmail.com
    
    File: task.h
*/


#ifndef TASK_H
#define TASK_H


//#define TASK_DEBUG

#define DEFAULT_STACK 0x200


/* Task states */
#define TS_READY 	1
#define TS_SLEEP 	2
#define TS_BLOCKED 	3


/* Some numeric constants */
#define INITIAL_R14     0x0000000e
#define INITIAL_R12     0x0000000c
#define INITIAL_R11     0x0000000b
#define INITIAL_R10     0x0000000a
#define INITIAL_R09     0x00000009
#define INITIAL_R08     0x00000008
#define INITIAL_R07     0x00000007
#define INITIAL_R06     0x00000006
#define INITIAL_R05     0x00000005
#define INITIAL_R04     0x00000004
#define INITIAL_R03     0x00000003
#define INITIAL_R02     0x00000002
#define INITIAL_R01     0x00000001
#define INITIAL_FLAGS   0x0000001f


/* Definition of a task */
typedef struct task_type
{
    dword sp;                               // Stack pointer - 4 bytes

    struct task_type *next;                // Pointer to next task - 4 bytes

    dword *stack;                          // Pointer to stack data - 4 bytes
    dword tid;

    dword stack_size;                       // Size fo stack - 4 bytes

	dword sleep;                            // Sleep timer - 4 bytes
    
    dword wait_signal;
    
    byte state;                             // Current state 1 byte
    
} task_type;



void task_init( void );

byte task_create(void *process, dword stack_size, dword param);

void task_schedule( void );

void task_reschedule( void );

void show_task(char *name, task_type *t);

void task_sleep(unsigned int t);


void task_swi_isr( void ) __attribute__((naked));       // ISR for software int
void task_tick_isr( void ) __attribute__((naked));      // ISR for timer




/* Accessors */
dword task_get_tasks( void );
dword task_get_tid( void );
task_type * task_get_current( void );



extern dword core_stack_size;       // User stack size from linker script
extern dword core_stack_addr;       // Address of user stack from linked script

#endif

