/*
    YoctOS: LPC2148 Dev Board Operating System

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.        

    (C) Jason Hunt
    nulluser@gmail.com
   
    File: printk.c
*/

#include <stdarg.h>

#include <core.h>


/* Output an integer to the emit function */
static void device_print_int(void emit_f(byte), dword n, dword base)
{
    byte chars[] = "0123456789ABCDEF";

    if (n == 0) 
    { 
        emit_f('0'); 
        return; 
    }

    dword i = 0;

    byte digits[12];
               
    while(n) 
    { 
        digits[i++] = n % base; 
        n /= base; 
    }
   
    while(i)   
        emit_f(chars[digits[i-- -1]]);
}
/* End of device_print_int */


/* The monster */
static void device_printf(const char *fmt, void emit_f(byte), va_list args)
{
    while(*fmt)
    {
        if (*fmt=='\\')
        {
            fmt++;
  
            if (*fmt == 'n') emit_f('\n'); else
            if (*fmt == '\\') emit_f('\\');
           
        } else
        if (*fmt=='%')
        {
            fmt++;

            // Decode format char
            if (*fmt == 'c')
            {
                fmt++;
                emit_f(va_arg(args, int)); 
            } else
            if (*fmt == 'd')
            {
                fmt++;
                device_print_int(emit_f, va_arg(args, dword), 10);
            } else
            if (*fmt == 's')
            {
                fmt++;
                char *s = va_arg(args, char*);
                while(*s) emit_f(*s++);
            } else
            if (*fmt == 'x')
            {
                fmt++;
                device_print_int(emit_f,  va_arg(args, dword), 16);
            }         
        } else
            emit_f(*fmt++);
    }
}
/* End of device_printf */



/* Print formetted string to serial port */
void printk(char *fmt, ...)
{
    va_list args;
    va_start(args, fmt);

    device_printf(fmt, serial_tx, args);

    va_end(args);
}
/* End of printk */



/* Print formetted string to LCD */
void video_printk(byte x, byte y, char *fmt, ...)
{
    va_list args;
    va_start(args, fmt);

    byte tm_x, tm_y;


    static dword l = 0;

    if (l == 0) l = signal_create(1);

    signal_wait(l);

    video_get_pos(&tm_x, &tm_y);

    video_set_pos(x, y);

    device_printf(fmt, video_putch, args);

    video_set_pos(tm_x, tm_y);
 
    signal_fire(l);

    va_end(args);
}
/* End of lcd_printk */




/* Print formetted string to serial port */
void panic(char *fmt, ...)
{
    va_list args;
    va_start(args, fmt);

    device_printf(fmt, serial_tx,  args);

    va_end(args);

    while(1);
}
/* End of printk */



