/*
    YoctOS: LPC2148 Dev Board Operating System

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.        

    (C) Jason Hunt
    nulluser@gmail.com
       
    File: signal.h
*/

#include <core.h>

static signal_type *signals = NULL;

static dword next_signal = 1;


/* Setup signals 
  Dependencies: None.
*/
void signal_init(void)
{
    

}
/* End of signal_init */




/* Create a signal */
dword signal_create(dword count)
{
    signal_type *s = (signal_type *)kmalloc(sizeof(signal_type));
    
    if (s == NULL) return(0);
        
    s->next = signals;
    s->id = next_signal++;
    s->count = count;

//    *s_id = s->id;
    
    signals = s;
    
    
    return(s->id);
}
/* End of signal_create */




/* Delete a signal */
byte signal_delete(dword s_id)
{   
    signal_type *p = signals, *s = signals;

    while(s != NULL && s->id != s_id)
    {
        p = s;
        s = s -> next;
    }
    
    if (s == NULL) return(FAIL);
    
    p->next = s->next;
    kfree(s);
    
    
    return(OK);  
}
/* End of signal_delete */



/* Wait on a signal */
byte signal_wait(dword s_id)
{
    enter_critical();

    signal_type *s = signals;

    // Find the signal
    while(s != NULL && s->id != s_id)
        s = s -> next;

    if (s == NULL) { leave_critical(); return(FAIL);     ;}

   // printk("Waiting on %d val: %d\n", s_id, s->count);

    if (--s->count < 0)
    {
        task_type *cur_task = task_get_current();

        cur_task->state = TS_BLOCKED;
        
        cur_task->wait_signal = s_id;


       // video_printk(1, 10, "Blocked %d\n", cur_task->tid);

        leave_critical();
        task_reschedule();
        return(OK);

    }   

    //printk("Done Waiting on %d val: %d\n", s_id, s->count);

    leave_critical();
    return(OK);
}
/* End of signal_wait */





/* Activate a signal */
byte signal_fire(dword s_id)
{
    
    enter_critical();

    signal_type *s = signals;

    while(s != NULL && s->id != s_id)
        s = s -> next;

    if (s == NULL) { leave_critical();  return(FAIL);    }

   // printk("Signaled %d val: %d\n", s_id, s->count);

//    if (++(s->count) >= 0)
    if (s->count++ < 0)

    {
      //  printk("Wake up\n");

//        extern task_type *cur_task;

        // Wake all processes waiting on s_is
        task_type *t = task_get_current();//cur_task ;
        
        dword i = task_get_tasks();
    
        byte found = 0;
    
    
        while(i-- && !found)
        {
            if (t->wait_signal == s_id)
            {
            //    printk("Firing up %d\n", t->tid);

                t->wait_signal = 0;
                t->state = TS_READY;
                found = 1;
            }

            t=t->next;
            
        }

        leave_critical();
   
      //  task_reschedule();
    
        return(OK);

    }   

  // printk("11 Signaled %d val: %d\n", s_id, s->count);

    leave_critical();
    
    return(OK);
}
/* End of signal_fire */



/*byte signal_fire2(dword s_id)
{
    
    enter_critical();

    signal_type *s = signals;

    while(s != NULL && s->id != s_id)
    {
        s = s -> next;
    }

    if (s == NULL) { leave_critical();  return(FAIL);    }


   // printk("Signaled %d val: %d\n", s_id, s->count);



    if (++(s->count) >= 0)
    {
      //  printk("Wake up\n");

        extern task_type *cur_task;

        // Wake all processes waiting on s_is
        task_type *t = cur_task ;
        
        dword i = task_get_tasks();
    
        byte found = 0;
    
    
        while(i-- && !found)
        {
            if (t->wait_signal == s_id)
            {
            //    printk("Firing up %d\n", t->tid);

                t->wait_signal = 0;
                t->state = TS_READY;
                found = 1;
            }

            t=t->next;
            
        }

        leave_critical();
   
      //  task_reschedule();
    
        return(OK);


    }   

  // printk("11 Signaled %d val: %d\n", s_id, s->count);

    leave_critical();
    
    
    return(OK);
    
}*/
/* End of signal_fire */




