/*
    YoctOS: LPC2148 Dev Board Operating System
        
    (C) Jason Hunt
    nulluser@gmail.com
        
    File: ram_link.ld
*/

ENTRY(_boot)

/* specify the LPC2148 memory areas  */

MEMORY 
{
	flash    : ORIGIN = 0,          LENGTH = 512K	
	isp1(A)  : ORIGIN = 0x40000120, LENGTH = 224	
	ram      : ORIGIN = 0x40000200, LENGTH = 32513  
	isp2(A)  : ORIGIN = 0x40007FE0, LENGTH = 32
	usb      : ORIGIN = 0x7FD00000, LENGTH = 8192  
}


/* Setup stack sizes */
stack_end = 0x40007EDC;

undef_stack = 0x0040;
abort_stack = 0x0040;
fiq_stack   = 0x0080;
irq_stack   = 0x0200;
svc_stack   = 0x0080;
user_stack  = 0x0300;


 /* size of all stacks */
stack_size =  undef_stack + abort_stack + fiq_stack;
stack_size += irq_stack   + svc_stack   + user_stack; 

core_stack_addr = stack_end - stack_size + user_stack; /* Top of user stack */
core_stack_size = user_stack;


        
SECTIONS 
{
	. = 0;					/* set location counter to address zero  */

	.text :					/* collect all sections that should go into FLASH after startup  */ 
	{
        *(.startup)
		*(.text)			/* all .text sections (code)  */
		*(.rodata)			/* all .rodata sections (constants, strings, etc.)  */
		*(.rodata*)			/* all .rodata* sections (constants, strings, etc.)  */
		*(.glue_7)			/* all .glue_7 sections  (no idea what these are) */
		*(.glue_7t)			/* all .glue_7t sections (no idea what these are) */
		_etext = .;			/* define a global symbol _etext just after the last code byte */
	} >ram			
	
	. = ALIGN(4);			/* advance location counter to the next 32-bit boundary */

	.data :			  		/* collect all initialized .data sections that go into RAM  */ 
	{
		_data = .;			/* create a global symbol marking the start of the .data section  */
		*(.data)			/* all .data sections  */
		_edata = .;			/* define a global symbol marking the end of the .data section  */
	} >ram 					/* put all the above into RAM (but load the LMA copy into FLASH) */

/*	} >ram AT >flash*/	

							/* put all the above into RAM (but load the LMA copy into FLASH) */

	. = ALIGN(4);			/* advance location counter to the next 32-bit boundary */

	.bss :					/* collect all uninitialized .bss sections that go into RAM  */
	{
		_bss_start = .;		/* define a global symbol marking the start of the .bss section */
		*(.bss)				/* all .bss sections  */
	} >ram					/* put all the above in RAM (it will be cleared in the startup code */

	. = ALIGN(4);			/* advance location counter to the next 32-bit boundary */
	_bss_end = . ;			/* define a global symbol marking the end of the .bss section */
	
	
	heap_start = .;       /* Start of heap ram */
	
	heap_end = stack_end - stack_size;
	
	
}
	_end = .;				/* define a global symbol marking the end of application RAM */
	
